use super::{ParseError, Rule};
use pest::iterators::{Pair, Pairs};
use pest::pratt_parser::{Assoc, Op, PrattParser};

#[derive(Debug, Clone)]
pub enum Expr {
    //Empty
    Empty,
    // Literals
    Number(f64),
    String(String),
    Boolean(bool),
    None,
    Undefined,
    Underscore,

    // Variables and calls
    Identifier(String),
    MemberAccess(Box<Expr>, String),
    Index(Box<Expr>, Box<Expr>),
    Call(Box<Expr>, Vec<Expr>),

    // Operators
    Binary(BinaryOp, Box<Expr>, Box<Expr>),
    Unary(UnaryOp, Box<Expr>),
    Ternary(Box<Expr>, Box<Expr>, Box<Expr>),
    Assignment(String, Box<Expr>),

    // Control flow
    If(Box<Expr>, Box<Block>, Option<Box<Block>>),
    Match(Option<Box<Expr>>, Vec<MatchArm>),

    // Collections
    Array(Vec<Expr>),

    // Postfix
    PostIncrement(Box<Expr>),
    PostDecrement(Box<Expr>),
}

#[derive(Debug, Clone)]
pub enum BinaryOp {
    Add,
    Sub,
    Mul,
    Div,
    Mod,
    Eq,
    Ne,
    Lt,
    Gt,
    Le,
    Ge,
    And,
    Or,
    RangeInclusive,
    RangeExclusive,
    Is,
}

#[derive(Debug, Clone)]
pub enum UnaryOp {
    Not,
    Neg,
    PreIncrement,
    PreDecrement,
}

#[derive(Debug, Clone)]
pub struct MatchArm {
    pattern: MatchPattern,
    body: Expr,
}

#[derive(Debug, Clone)]
pub enum MatchPattern {
    Wildcard,
    Expression(Expr),
    Condition(Vec<(BinaryOp, Expr)>),
}

#[derive(Debug, Clone)]
pub struct Block {
    statements: Vec<Statement>,
}

#[derive(Debug, Clone)]
pub enum ReturnType {
    Simple(Type),
    Named {
        name: String,
        type_annotation: Type,
        default_value: Option<Expr>,
    },
}

#[derive(Debug, Clone)]
pub enum Statement {
    Import {
        name: String,
        from: String,
    },
    Function {
        name: String,
        params: Vec<Param>,
        return_type: Option<ReturnType>,
        extends: Option<String>,
        body: Block,
    },
    Variable {
        kind: VarKind,
        name: String,
        type_annotation: Option<Type>,
        value: Expr,
    },
    Defer {
        condition: Option<Expr>,
        binding: Option<String>,
        body: Block,
    },
    Watch {
        target: String,
        body: Block,
    },
    Return(Option<Expr>),
    If(Expr, Box<Statement>, Option<Box<Statement>>),
    For {
        var: String,
        index: Option<String>,
        iterable: Expr,
        body: Box<Statement>,
    },
    While(Expr, Box<Statement>),
    Expression(Expr),
    Block(Block),
}

#[derive(Debug, Clone)]
pub enum VarKind {
    Const,
    Var,
    Live,
}

#[derive(Debug, Clone)]
pub struct Param {
    name: String,
    type_annotation: Option<Type>,
}

#[derive(Debug, Clone)]
pub enum Type {
    Primitive(String),
    Array(Box<Type>),
    Optional(Box<Type>),
    ErrorUnion(Option<String>, Box<Type>),
    Named(String),
}

pub struct SolacePrattParser {
    pratt: PrattParser<Rule>,
}

impl SolacePrattParser {
    pub fn new() -> Self {
        let pratt = PrattParser::new()
            .op(Op::infix(Rule::assign, Assoc::Right))
            .op(Op::infix(Rule::question, Assoc::Right) | Op::infix(Rule::colon, Assoc::Right))
            .op(Op::infix(Rule::or, Assoc::Left))
            .op(Op::infix(Rule::and, Assoc::Left))
            .op(Op::infix(Rule::eq, Assoc::Left)
                | Op::infix(Rule::ne, Assoc::Left)
                | Op::infix(Rule::is_kw, Assoc::Left))
            .op(Op::infix(Rule::lt, Assoc::Left)
                | Op::infix(Rule::gt, Assoc::Left)
                | Op::infix(Rule::le, Assoc::Left)
                | Op::infix(Rule::ge, Assoc::Left))
            .op(Op::infix(Rule::range_inclusive, Assoc::Left)
                | Op::infix(Rule::range_exclusive, Assoc::Left))
            .op(Op::infix(Rule::plus, Assoc::Left) | Op::infix(Rule::minus, Assoc::Left))
            .op(Op::infix(Rule::multiply, Assoc::Left)
                | Op::infix(Rule::divide, Assoc::Left)
                | Op::infix(Rule::modulo, Assoc::Left))
            .op(Op::prefix(Rule::not)
                | Op::prefix(Rule::minus)
                | Op::prefix(Rule::increment)
                | Op::prefix(Rule::decrement))
            .op(Op::postfix(Rule::increment) | Op::postfix(Rule::decrement));

        SolacePrattParser { pratt }
    }

    pub fn parse_expr(&self, pairs: Pairs<Rule>) -> Result<Expr, ParseError> {
        if pairs.clone().count() == 0 {
            return Ok(Expr::Empty);
        }

        self.pratt
            .map_primary(|primary| self.parse_primary(primary))
            .map_infix(|lhs, op, rhs| self.parse_infix(lhs, op, rhs))
            .map_prefix(|op, rhs| self.parse_prefix(op, rhs))
            .map_postfix(|lhs, op| self.parse_postfix(lhs, op))
            .parse(pairs)
    }

    fn parse_primary(&self, pair: Pair<Rule>) -> Result<Expr, ParseError> {
        match pair.as_rule() {
            Rule::number_literal => {
                let num = pair
                    .as_str()
                    .parse::<f64>()
                    .map_err(|_| ParseError::InvalidNumber(pair.as_rule()))?;
                Ok(Expr::Number(num))
            }
            Rule::string_literal => {
                let s = pair.as_str();
                Ok(Expr::String(s[1..s.len() - 1].to_string()))
            }
            Rule::boolean_literal => Ok(Expr::Boolean(pair.as_str() == "true")),
            Rule::none_kw => Ok(Expr::None),
            Rule::undefined_kw => Ok(Expr::Undefined),
            Rule::underscore => Ok(Expr::Underscore),
            Rule::identifier => Ok(Expr::Identifier(pair.as_str().to_string())),
            Rule::array_literal => {
                let mut elements = vec![];
                for inner in pair.into_inner() {
                    if inner.as_rule() == Rule::expression {
                        elements.push(self.parse_expr(inner.into_inner())?);
                    }
                }
                Ok(Expr::Array(elements))
            }
            Rule::if_expr => self.parse_if_expr(pair),
            Rule::match_expr => self.parse_match_expr(pair),
            _ => Err(ParseError::UnknownPrimary(pair.as_rule())),
        }
    }

    fn parse_infix(
        &self,
        lhs: Result<Expr, ParseError>,
        op: Pair<Rule>,
        rhs: Result<Expr, ParseError>,
    ) -> Result<Expr, ParseError> {
        let lhs = lhs?;
        let rhs = rhs?;

        match op.as_rule() {
            Rule::plus => Ok(Expr::Binary(BinaryOp::Add, Box::new(lhs), Box::new(rhs))),
            Rule::minus => Ok(Expr::Binary(BinaryOp::Sub, Box::new(lhs), Box::new(rhs))),
            Rule::multiply => Ok(Expr::Binary(BinaryOp::Mul, Box::new(lhs), Box::new(rhs))),
            Rule::divide => Ok(Expr::Binary(BinaryOp::Div, Box::new(lhs), Box::new(rhs))),
            Rule::modulo => Ok(Expr::Binary(BinaryOp::Mod, Box::new(lhs), Box::new(rhs))),
            Rule::eq => Ok(Expr::Binary(BinaryOp::Eq, Box::new(lhs), Box::new(rhs))),
            Rule::ne => Ok(Expr::Binary(BinaryOp::Ne, Box::new(lhs), Box::new(rhs))),
            Rule::lt => Ok(Expr::Binary(BinaryOp::Lt, Box::new(lhs), Box::new(rhs))),
            Rule::gt => Ok(Expr::Binary(BinaryOp::Gt, Box::new(lhs), Box::new(rhs))),
            Rule::le => Ok(Expr::Binary(BinaryOp::Le, Box::new(lhs), Box::new(rhs))),
            Rule::ge => Ok(Expr::Binary(BinaryOp::Ge, Box::new(lhs), Box::new(rhs))),
            Rule::and => Ok(Expr::Binary(BinaryOp::And, Box::new(lhs), Box::new(rhs))),
            Rule::or => Ok(Expr::Binary(BinaryOp::Or, Box::new(lhs), Box::new(rhs))),
            Rule::is_kw => Ok(Expr::Binary(BinaryOp::Is, Box::new(lhs), Box::new(rhs))),
            Rule::range_inclusive => Ok(Expr::Binary(
                BinaryOp::RangeInclusive,
                Box::new(lhs),
                Box::new(rhs),
            )),
            Rule::range_exclusive => Ok(Expr::Binary(
                BinaryOp::RangeExclusive,
                Box::new(lhs),
                Box::new(rhs),
            )),
            Rule::assign => {
                if let Expr::Identifier(name) = lhs {
                    Ok(Expr::Assignment(name, Box::new(rhs)))
                } else {
                    Err(ParseError::InvalidLeftHand(lhs))
                }
            }
            Rule::question => {
                // Handle ternary - need to parse the rest
                // This is simplified - in practice you'd need more complex handling
                Ok(Expr::Ternary(
                    Box::new(lhs),
                    Box::new(rhs),
                    Box::new(Expr::None),
                ))
            }
            _ => Err(ParseError::UnknownInfixOperator(op.as_rule())),
        }
    }

    fn parse_prefix(
        &self,
        op: Pair<Rule>,
        rhs: Result<Expr, ParseError>,
    ) -> Result<Expr, ParseError> {
        let rhs = rhs?;

        match op.as_rule() {
            Rule::not => Ok(Expr::Unary(UnaryOp::Not, Box::new(rhs))),
            Rule::minus => Ok(Expr::Unary(UnaryOp::Neg, Box::new(rhs))),
            Rule::increment => Ok(Expr::Unary(UnaryOp::PreIncrement, Box::new(rhs))),
            Rule::decrement => Ok(Expr::Unary(UnaryOp::PreDecrement, Box::new(rhs))),
            _ => Err(ParseError::UnknownPrefixOperator(op.as_rule())),
        }
    }

    fn parse_postfix(
        &self,
        lhs: Result<Expr, ParseError>,
        op: Pair<Rule>,
    ) -> Result<Expr, ParseError> {
        let lhs = lhs?;

        match op.as_rule() {
            Rule::increment => Ok(Expr::PostIncrement(Box::new(lhs))),
            Rule::decrement => Ok(Expr::PostDecrement(Box::new(lhs))),
            _ => Err(ParseError::UnknownPostfixOperator(op.as_rule())),
        }
    }

    fn parse_if_expr(&self, pair: Pair<Rule>) -> Result<Expr, ParseError> {
        let mut inner = pair.into_inner();
        let condition = self.parse_expr(inner.next().unwrap().into_inner())?;
        let then_block = self.parse_block(inner.next().unwrap())?;
        let else_block = inner.next().map(|p| self.parse_block(p)).transpose()?;

        Ok(Expr::If(
            Box::new(condition),
            Box::new(then_block),
            else_block.map(Box::new),
        ))
    }

    fn parse_match_expr(&self, pair: Pair<Rule>) -> Result<Expr, ParseError> {
        let mut inner = pair.into_inner();
        let target = if let Some(p) = inner.peek() {
            if p.as_rule() == Rule::expression {
                Some(Box::new(
                    self.parse_expr(inner.next().unwrap().into_inner())?,
                ))
            } else {
                None
            }
        } else {
            None
        };

        let mut arms = vec![];
        for arm_pair in inner {
            if arm_pair.as_rule() == Rule::match_arm {
                arms.push(self.parse_match_arm(arm_pair)?);
            }
        }

        Ok(Expr::Match(target, arms))
    }

    fn parse_match_arm(&self, pair: Pair<Rule>) -> Result<MatchArm, ParseError> {
        let mut inner = pair.into_inner();
        let pattern = self.parse_match_pattern(inner.next().unwrap())?;
        let body = self.parse_expr(inner.next().unwrap().into_inner())?;

        Ok(MatchArm { pattern, body })
    }

    fn parse_match_pattern(&self, pair: Pair<Rule>) -> Result<MatchPattern, ParseError> {
        // Simplified pattern parsing
        match pair.as_rule() {
            Rule::underscore => Ok(MatchPattern::Wildcard),
            _ => Ok(MatchPattern::Expression(
                self.parse_expr(pair.into_inner())?,
            )),
        }
    }

    fn parse_block(&self, pair: Pair<Rule>) -> Result<Block, ParseError> {
        let mut statements = vec![];
        for stmt in pair.into_inner() {
            statements.push(self.parse_statement(stmt)?);
        }
        Ok(Block { statements })
    }

    fn parse_statement(&self, pair: Pair<Rule>) -> Result<Statement, ParseError> {
        match pair.as_rule() {
            Rule::expression_stmt => {
                let expr = self.parse_expr(pair.into_inner())?;
                Ok(Statement::Expression(expr))
            }
            Rule::return_stmt => {
                let mut inner = pair.into_inner();
                let expr = inner
                    .next()
                    .map(|p| self.parse_expr(p.into_inner()))
                    .transpose()?;
                Ok(Statement::Return(expr))
            }
            Rule::function_decl => {
                let decl = self.parse_function_decl(pair.into_inner())?;
                Ok(decl)
            }
            // Add other statement parsing here
            _ => Err(ParseError::UnknownStatement(pair.as_rule())),
        }
    }

    fn parse_type(&self, pair: Pair<Rule>) -> Result<Type, ParseError> {
        match pair.as_rule() {
            Rule::type_annotation => {
                // Type annotation starts with colon, skip it
                let mut inner = pair.into_inner();
                inner.next(); // skip colon
                self.parse_type_expr(inner.next().unwrap())
            }
            Rule::type_expr => self.parse_type_expr(pair),
            _ => Err(ParseError::ShouldBeType(pair.as_rule())),
        }
    }

    fn parse_type_expr(&self, pair: Pair<Rule>) -> Result<Type, ParseError> {
        let inner = pair.into_inner();
        let mut current_type = None;
        let mut is_optional = false;
        let mut error_type = None;
        let mut array_depth = 0;

        for part in inner {
            match part.as_rule() {
                Rule::optional_prefix => {
                    is_optional = true;
                }
                Rule::error_union_prefix => {
                    // Could be just "!", which defaults to Error!, or "ErrorType!"
                    let prefix_inner = part.into_inner();
                    if let Some(error_name) = prefix_inner.peek() {
                        error_type = Some(error_name.as_str().to_string());
                    } else {
                        error_type = Some("Error".to_string());
                    }
                }
                Rule::base_type => {
                    current_type = Some(self.parse_base_type(part)?);
                }
                Rule::array_suffix => {
                    array_depth += 1;
                }
                _ => {}
            }
        }

        let mut result_type = current_type.ok_or(ParseError::MissingBaseType())?;

        // Apply array suffixes
        for _ in 0..array_depth {
            result_type = Type::Array(Box::new(result_type));
        }

        // Apply error union
        if let Some(err_type) = error_type {
            result_type = Type::ErrorUnion(Some(err_type), Box::new(result_type));
        }

        // Apply optional
        if is_optional {
            result_type = Type::Optional(Box::new(result_type));
        }

        Ok(result_type)
    }

    fn parse_base_type(&self, pair: Pair<Rule>) -> Result<Type, ParseError> {
        let mut inner = pair.into_inner();
        let type_part = inner.next().unwrap();

        match type_part.as_rule() {
            Rule::primitive_type => Ok(Type::Primitive(type_part.as_str().to_string())),
            Rule::identifier => Ok(Type::Named(type_part.as_str().to_string())),
            _ => Err(ParseError::UnknownTypePart(type_part.as_rule())),
        }
    }

    fn parse_param_list(&self, pair: Pair<Rule>) -> Result<Vec<Param>, ParseError> {
        let mut params = vec![];

        for inner in pair.into_inner() {
            if inner.as_rule() == Rule::param {
                params.push(self.parse_param(inner)?);
            }
        }

        Ok(params)
    }

    fn parse_param(&self, pair: Pair<Rule>) -> Result<Param, ParseError> {
        let mut inner = pair.into_inner();
        let name = inner.next().unwrap().as_str().to_string();

        let type_annotation = if let Some(type_pair) = inner.next() {
            Some(self.parse_type(type_pair)?)
        } else {
            None
        };

        Ok(Param {
            name,
            type_annotation,
        })
    }

    fn parse_return_type(&self, pair: Pair<Rule>) -> Result<ReturnType, ParseError> {
        let mut inner = pair.into_inner();
        let wrapped = inner.next().unwrap();

        // Skip intermediate rules
        let first = match wrapped.as_rule() {
            Rule::return_type_simple => wrapped.into_inner().next().unwrap(),
            Rule::return_type_named => wrapped.into_inner().next().unwrap(),
            _ => wrapped,
        };

        match first.as_rule() {
            // Named return variable: fn foo() => (name: Type = default?) {/*...*/}
            Rule::arrow_fat => {
                let name = inner.next().unwrap().as_str().to_string();
                let type_annotation = self.parse_type(inner.next().unwrap())?;
                let default_value = if let Some(expr_pair) = inner.next() {
                    Some(self.parse_expr(expr_pair.into_inner())?)
                } else {
                    None
                };

                Ok(ReturnType::Named {
                    name,
                    type_annotation,
                    default_value,
                })
            }
            // Simple return type: fn foo(): Type {/*...*/}
            Rule::type_annotation => {
                let type_annotation = self.parse_type(first)?;
                Ok(ReturnType::Simple(type_annotation))
            }
            // Brother ewww... whats that?
            _ => Err(ParseError::UnknownReturnType(
                first.as_str().to_owned(),
                first.as_rule(),
            )),
        }
    }

    fn parse_function_decl(&self, mut pairs: Pairs<Rule>) -> Result<Statement, ParseError> {
        // Skip 'fn' keyword
        pairs.next();

        let name = pairs.next().unwrap().as_str();
        let params = self.parse_param_list(pairs.next().unwrap())?;

        let mut extends = None;
        let mut return_type = None;
        let mut body = None;

        while let Some(remaining) = pairs.next() {
            match remaining.as_rule() {
                Rule::extends_kw => {
                    extends = Some(pairs.next().unwrap().as_str().to_string());
                }
                Rule::return_type => {
                    return_type = Some(self.parse_return_type(remaining)?);
                }
                Rule::block => {
                    body = Some(self.parse_block(remaining)?);
                }
                _ => {}
            }
        }

        Ok(Statement::Function {
            name: name.to_owned(),
            params,
            return_type,
            extends,
            body: body.ok_or(ParseError::MissingFunctionBody(name.to_owned()))?,
        })
    }
}

#[derive(Debug, Clone)]
pub struct Program {
    pub statements: Vec<Statement>,
}

impl Program {
    pub fn from_pairs(pairs: Pairs<Rule>) -> Result<Self, ParseError> {
        let parser = SolacePrattParser::new();
        let mut statements = Vec::new();

        for pair in pairs {
            if pair.as_rule() == Rule::program {
                for stmt_pair in pair.into_inner() {
                    if stmt_pair.as_rule() != Rule::EOI {
                        let stmt = parser.parse_statement(stmt_pair)?;
                        statements.push(stmt);
                    }
                }
            }
        }

        Ok(Program { statements })
    }
}
